/*
 * This file is part of jPlot
 *
 * jPlot is a Java version of Tim Shepard's xplot which is distributed 
 * under the M.I.T. license (as below).
 *
 * Tim Shepard's original xplot is an amazing piece of software. It was
 * designed to be extraordinarily fast and efficient and as such, it has
 * only a very thin user interface. Those design goals, however, make
 * xplot extremely difficult to port to new architectures and also make it
 * difficult to use for the casual user. jPlot was designed to address
 * these limitations while keeping as much of the original efficiency as
 * possible. We thank Tim Shepard for his hard work on xplot and hope that
 * jPlot can live up to his high standards.
 *
 * jPlot was developed by Avinash Lakhiani at the Internetworking Research
 * Lab (IRG), Ohio University, as part of Dr. Shawn Ostermann's tcptrace
 * project. The main goal behind the development of jPlot was to make the 
 * plotting tool used with tcptrace more portable. As a result, jPlot
 * retains many of the original algorithms implemented in xplot, and as per
 * the terms and conditions of the M.I.T. license, continue to remain under
 * that license. The M.I.T license is as follows:
 *
 * --- M.I.T License Begin ---
 * 
 * Copyright 1992,1993 by the Massachusetts Institute of Technology.
 *                   All rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS", AND M.I.T. MAKES NO REPRESENTATIONS
 * OR WARRANTIES, EXPRESS OR IMPLIED.  By way of example, but not
 * limitation, M.I.T. MAKES NO REPRESENTATIONS OR WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE
 * OF THE LICENSED SOFTWARE OR DOCUMENTATION WILL NOT INFRINGE ANY THIRD
 * PARTY PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.
 *
 * The name of the Massachusetts Institute of Technology or M.I.T. may
 * NOT be used in advertising or publicity pertaining to distribution of
 * the software.  Title to copyright in this software and any associated
 * documentation shall at all times remain with M.I.T., and USER agrees
 * to preserve same.
 * 
 * --- M.I.T. License End ---
 * 
 * jPlot also has a lot of added features that are not part of the original
 * xplot code. Those sections of code are distributed under the following
 * BSD License:
 *
 * --- BSD License Begin ---
 *
 * Copyright (c) 2002, 2003 by Avinash Lakhiani
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *	
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution. 
 *   - Neither the name of Ohio University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission. 
 *	   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * --- BSD License End ---
 * 
 * You should have received a file named "DISTINCTION" along with this
 * distribution, that clearly identifies all the algorithms that were
 * retained from the original xplot code, and remain under the M.I.T.
 * license. The rest of the code was written for jPlot, and falls under
 * the BSD license.
 * 
 * jPlot is free software; you can redistribute it and/or modify it under the
 * terms and conditions of both the above licenses.
 *
 * jPlot is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the above licenses for more details.
 *
 * Author:  Avinash Lakhiani
 *          Internetworking Research Group
 *          Ohio University
 *          Athens, OH
 *          avinash.lakhiani@ohiou.edu
 *          http://www.tcptrace.org/jPlot
 *
 */

/*
 * $Id: Plotter.java,v 1.2 2002/12/04 01:58:52 alakhian Exp $
 */ 

/* Import Packages */
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.io.*;
import java.lang.String;
import java.util.List;

/* Class       - Plotter
 * Description - this class holds all the attributes for a plotter, and all its commands
 * Extends     - NULL
 * Implements  - NULL
 */
public class Plotter
{
   private String plotFile;          // plotter file to read the commands from
   private boolean isMargin;         // Indicates if an extra margin is required around the plot
   protected List commands;          // List of plotting commands read from the file
   protected int xType;              // x-axis type
   protected int yType;              // y-axis type
   protected String xUnits;          // x-axis units
   protected String yUnits;          // y-axis units
   protected double aspectRatio;     // Aspect ration, if specified in the file, else set to 0.0
   protected int viewNo;             // view number, used for zooming
   protected Coord xLeft[];          // left boundary of the particular view
   protected Coord xRight[];         // right boundary of the particular view 
   protected Coord yTop[];           // top boundary of the particular view
   protected Coord yBottom[];        // bottom boundary of the particular view
   protected Point2D origin;         // origin of this plot
   protected Point2D size;           // set size of the viewport
   protected Point2D mainSize;       // actual size of the viewport
   protected Point dragStart;        // mouse drag start point
   protected Point dragEnd;          // mouse drag end point
   protected Color defaultColor;     // default plotting color {set to black in constructor, changed if specified in file}
   protected Color currentColor;     // current plotting color {changes with specification on each line}
   protected Color foregroundColor;  // plotter foreground color
   protected Color backgroundColor;  // plotter background color
   protected int state;              // plotter state  

   /* Method      - Plotter (default constructor)
    * Parameters  - plotter filename
    * Returns     - NULL
    * Description - sets the plotter filename
    */
   public Plotter(String file, boolean isMargin)
     {
	/* Initializations */
	plotFile        = new String(file);
	commands        = new ArrayList();
	xUnits          = new String("");
	yUnits          = new String("");
	aspectRatio     = 0.0;
	viewNo          = 0;
	xLeft           = new Coord[Constant.NUMVIEWS];
	yBottom         = new Coord[Constant.NUMVIEWS];
	xRight          = new Coord[Constant.NUMVIEWS];
	yTop            = new Coord[Constant.NUMVIEWS];
	origin          = new Point2D.Double(Constant.DEFAULT_ORIGIN_X, Constant.DEFAULT_ORIGIN_Y);
	mainSize        = new Point2D.Double(0.0, 0.0);
	size            = new Point2D.Double(0.0, 0.0);	
	dragStart       = new Point(0, 0);
	dragEnd         = new Point(0, 0);
	defaultColor    = Color.white;
	currentColor    = Color.white;
	foregroundColor = Color.white;
	backgroundColor = Color.black;
	state           = State.NORMAL;
	this.isMargin   = isMargin;
	/* Read the commands from the file */
	readInput();
     
     }

   /* Method      - readInput
    * Parameters  - NULL
    * Returns     - NULL
    * Description - opens the specified input file and reads the plotter
    * arributes and commands.
    */
   private int readInput()
     {
	BufferedReader input = null;
	String line;
	int lineNo = 0;
	StringTokenizer st;
	
	try
	  {
	     input = new BufferedReader(new FileReader(plotFile));
	  }
	catch(FileNotFoundException e)
	  {
	     System.out.println(e.toString());
	     System.exit(1);
	  }
	
	try
	  {
	     /* Process commands */

	     /* Declarations & Initializations */
	     int nTokens = 0;                           // Count of tokens in one line 
	     String token[] = {"", "", "", "", "", ""}; // tokens read in one line (max tokens relevant could be 6)
	     
	     /* Looking for the string "new_plotter".
	      * (allowing multiple instances of that string)
	      */ 
	     do 
	       {
		  line = input.readLine(); lineNo++;
		  if(line == null) parseError(lineNo, line, "EOF before first line of input");
		  st = new StringTokenizer(line);
		  nTokens = st.countTokens();
	       }
	     while ((nTokens == 1) && (st.nextToken().compareTo("new_plotter") == 0));
	     
	     /* Check if we possibly have the coordinate types in the next line */
	     if(nTokens != 2) parseError(lineNo, line, "invalid input format -- expecting coord type names");
	     
	     /* Process coordinate types */
	     xType = cType.toInt(st.nextToken());
	     yType = cType.toInt(st.nextToken());
	     if(xType < 0 || yType < 0) parseError(lineNo, line, "unknown coord type");
	     
	     /* Loop and process commands */
	     for( ; ; )
	       {
		  line = input.readLine(); lineNo++;
		  if(line == null) break;
		  st = new StringTokenizer(line);		       
		  nTokens = st.countTokens();

		  /* DEBUGGING
		   System.out.println(lineNo + " : " + line + " : " + nTokens);
		   */ 
		  
		  /* Look for blank lines */
		  if (nTokens == 0) continue;
		  
		  /* Look for empty command lines */
		  token[0] = new String(st.nextToken());
		  if(token[0].compareTo(";") == 0) continue;
		  
		  /* Look for default color alone on a line */
		  if(nTokens == 1)
		    {
		       Color c = parseColor(token[0]);
		       if(c != null)
			 {
			    currentColor = c;
			    continue;
			 }
		    }

		  /* Coordinate types may be specified after default color */
		  if(nTokens == 2)
		    {		  
		       token[1] = new String(st.nextToken());
		       if((xType == cType.toInt(token[0])) &&
			  (yType == cType.toInt(token[1]))
			  )
			 continue;			 
		    }
		     
		  /* Parsing the different commands in the input file */
		  if(token[0].compareToIgnoreCase("aspect_ratio") == 0)  // Type "aspect_ratio"
		    {
		       if(nTokens != 2) parseError(lineNo, line, "input format error");
		       if(xType != yType)
			 parseError(lineNo, line, "aspect_ratio requires identical coordinate types");
		       Double dObj = new Double(token[1]);
		       aspectRatio = dObj.doubleValue();
		    
		    }else
		    if(token[0].compareToIgnoreCase("x") == 0)  // Type "x"
		    {
		       Color c = currentColor;
		       Coord x1;
		       Coord y1;
		       if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
		       token[1] = new String(st.nextToken());
		       token[2] = new String(st.nextToken());
		       x1 = newCoord(xType, token[1]);
		       y1 = newCoord(yType, token[2]);
		       if(nTokens == 4)
			 {
			    token[3] = new String(st.nextToken());
			    c = parseColor(token[3]);
			 }
		    /* arguments for this constructor
                     * Command(type, position, mappped, decoration, color, coord1, coord2, text)
		     */ 
		       Command cmd = new Command(cmdType.X, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
		       commands.add(0, cmd);
		       
		    }else
		    if((token[0].compareToIgnoreCase(".") == 0)  || (token[0].compareToIgnoreCase("dot") == 0))// Type ". & dot"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.DOT, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if((token[0].compareToIgnoreCase("+") == 0) || (token[0].compareToIgnoreCase("plus") == 0))  // Type "+ & plus"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.PLUS, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("box") == 0)  // Type "box"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.BOX, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("diamond") == 0)  // Type "diamond"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.DIAMOND, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("utick") == 0)  // Type "utick"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.UTICK, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("dtick") == 0)  // Type "dtick"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.DTICK, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("ltick") == 0)  // Type "ltick"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.LTICK, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("rtick") == 0)  // Type "rtick"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.RTICK, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("htick") == 0)  // Type "htick"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.HTICK, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("vtick") == 0)  // Type "vtick"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.VTICK, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("uarrow") == 0)  // Type "uarrow"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.UARROW, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("darrow") == 0)  // Type "darrow"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.DARROW, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("larrow") == 0)  // Type "larrow"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.LARROW, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("rarrow") == 0)  // Type "rarrow"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.RARROW, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("line") == 0)  // Type "line"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 Coord x2;
			 Coord y2;
			 if((nTokens != 5) && (nTokens != 6)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 token[3] = new String(st.nextToken());
			 token[4] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 x2 = newCoord(xType, token[3]);
			 y2 = newCoord(yType, token[4]);
			 if(nTokens == 6)
			   {
			      token[5] = new String(st.nextToken());
			      c = parseColor(token[5]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.LINE, Position.UNDEFINED, false, false, c, x1, y1, x2, y2, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("dline") == 0)  // Type "dline"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 Coord x2;
			 Coord y2;
			 if((nTokens != 5) && (nTokens != 6)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 token[3] = new String(st.nextToken());
			 token[4] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 x2 = newCoord(xType, token[3]);
			 y2 = newCoord(yType, token[4]);
			 if(nTokens == 6)
			   {
			      token[5] = new String(st.nextToken());
			      c = parseColor(token[5]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.DLINE, Position.UNDEFINED, false, false, c, x1, y1, x2, y2, "");
			 commands.add(0, cmd);

		      }else
		    if(token[0].compareToIgnoreCase("title") == 0)  // Type "title"
		      {
			 String title = new String("");
			 /* The next line holds the title */
			 if(nTokens == 1)
			   {
			      title = input.readLine(); lineNo++;
			      if(title == null) break;
			   }
			 else if(nTokens == 2) title = new String(token[1]);
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.TITLE, Position.UNDEFINED, false, false, null, null, null, null, null, title);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("ctext") == 0)  // Type "ctext"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 String text = new String("");
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* The next line holds the text to be displayed at the specified location */
			 text = input.readLine(); lineNo++;
			 if(text == null) break;
			 
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.TEXT, Position.CENTERED, false, false, c, x1, y1, null, null, text);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("atext") == 0)  // Type "atext"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 String text = new String("");
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* The next line holds the text to be displayed at the specified location */
			 text = input.readLine(); lineNo++;
			 if(text == null) break;
			 
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.TEXT, Position.ABOVE, false, false, c, x1, y1, null, null, text);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("btext") == 0)  // Type "btext"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 String text = new String("");
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* The next line holds the text to be displayed at the specified location */
			 text = input.readLine(); lineNo++;
			 if(text == null) break;

			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.TEXT, Position.BELOW, false, false, c, x1, y1, null, null, text);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("ltext") == 0)  // Type "ltext"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 String text = new String("");
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* The next line holds the text to be displayed at the specified location */
			 text = input.readLine(); lineNo++;
			 if(text == null) break;

			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.TEXT, Position.LEFT, false, false, c, x1, y1, null, null, text);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("rtext") == 0)  // Type "rtext"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 String text = new String("");
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* The next line holds the text to be displayed at the specified location */
			 if(nTokens == 5) text = new String(st.nextToken());
			 else 
			   {
			      text = input.readLine(); lineNo++;
			      if(text == null) break;
			   }

			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.TEXT, Position.RIGHT, false, false, c, x1, y1, null, null, text);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("xlabel") == 0)  // Type "xlabel"
		      {
			 String xlabel = new String("");
			 /* The next line holds the xlabel */
			 if(nTokens == 1)
			   {
			      xlabel = input.readLine(); lineNo++;
			      if(xlabel == null) break;
			   }
			 else if(nTokens == 2) xlabel = new String(token[1]);
			      
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.XLABEL, Position.UNDEFINED, false, false, null, null, null, null, null, xlabel);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("ylabel") == 0)  // Type "ylabel"
		      {
			 String ylabel = new String("");
			 /* The next line holds the ylabel */
			 if(nTokens == 1)
			   {
			      ylabel = input.readLine(); lineNo++;
			      if(ylabel == null) break;
			   }
			 else if(nTokens == 2) ylabel = new String(token[1]);
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.YLABEL, Position.UNDEFINED, false, false, null, null, null, null, null, ylabel);
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("xunits") == 0)  // Type "xunits"
		      {
			 String xunits = new String("");
			 /* The next line holds the xunits */
			 if(nTokens == 1)
			   {
			      xunits = input.readLine(); lineNo++;
			      if(xunits == null) break;
			   }
			 else if(nTokens == 2) xunits = new String(token[1]);
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 xUnits = xunits;
			 
		      }else
		    if(token[0].compareToIgnoreCase("yunits") == 0)  // Type "yunits"
		      {
			 String yunits = new String("");
			 /* The next line holds the ylabel */
			 if(nTokens == 1)
			   {
			      yunits = input.readLine(); lineNo++;
			      if(yunits == null) break;
			   }
			 else if(nTokens == 2) yunits = new String(token[1]);
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 yUnits = yunits;
			 
		      }else
		    if(token[0].compareToIgnoreCase("invisible") == 0)  // Type "invisible"
		      {
			 Color c = currentColor;
			 Coord x1;
			 Coord y1;
			 if((nTokens != 3) && (nTokens != 4)) parseError(lineNo, line, "input format error");
			 token[1] = new String(st.nextToken());
			 token[2] = new String(st.nextToken());
			 x1 = newCoord(xType, token[1]);
			 y1 = newCoord(yType, token[2]);
			 if(nTokens == 4)
			   {
			      token[3] = new String(st.nextToken());
			      c = parseColor(token[3]);
			   }
			 /* arguments for this constructor
			  * Command(type, position, mappped, decoration, color, coord1, coord2, text)
			  */ 
			 Command cmd = new Command(cmdType.INVISIBLE, Position.UNDEFINED, false, false, c, x1, y1, null, null, "");
			 commands.add(0, cmd);
			 
		      }else
		    if(token[0].compareToIgnoreCase("go") == 0)  // Type "go"
		      {
			 System.out.println("Plotting file \"" + plotFile + "\"");
			 break;
		      }
		  else // Error
		    {
		       parseError(lineNo, line, "input format error"); 
		    }
		  
	       } // End of for-loop
	     
	     /* Close the input stream */
	     input.close();
	     
	     /* Initializing the window range parameters, they are currently */
	     for(int i = 0; i < Constant.NUMVIEWS; i++)
	       {
		  xLeft[i]   = newCoord(xType, "0");
		  xRight[i]  = newCoord(xType, "0");
		  yBottom[i] = newCoord(yType, "0");
		  yTop[i]    = newCoord(yType, "0");
	       }
	     
	     /* Computing the window range */
	     this.computeRange();
	     
	     /* Bump the coordinates. This is done in conjunction with the ticking algorithm. The step gets the
	      * correct range related to the axis ticks. (A bit complicated). {Algorithm from Tim Shepard's xplot}
	      */
	     this.bumpCoord(xType, xRight[viewNo]);
	     this.bumpCoord(yType, yTop[viewNo]);

	     /* DEBUGGING ... printing the commands list
	     int i = commands.size();
	     Command cmd;
	     while(i > 0)
	       {
		  cmd = (Command)commands.get(--i);
		  System.out.println(i + " : " + cmd.toString());
	       }
	      */ 
	     
	  }
	catch(NoSuchElementException e)
	  {
	     System.out.println(e.toString());
	  }
	catch(IndexOutOfBoundsException e)
	  {
	     System.out.println(e.toString());
	  }
        catch(IOException e)
	  {
	     System.out.println(e.toString());
	  }
	
	return(0);
	
     } // End of method readInput()

   /* Method      - parseError
    * Parameters  - line number, input line, error message
    * Returns     - NULL
    * Description - prints the error message for the error that occured during input parsing, 
    * along with the line number and input line.
    */
   private void parseError(int lineNo, String line, String error)
     {
	System.out.println("Plotter.java (file: "+ plotFile + "): in line number " +
			   lineNo + " : " + error + "\n\toffending line: " + line
			   );
	System.exit(1);
     
     } // End of method parseError()
	
   /* Method      - parseColor
    * Parameters  - color string
    * Returns     - result of operation {parsedcolor - successful, null - unsuccessful}
    * Description - parses the color string passed to it, and returns the appropriate java
    * color.
    */
   private Color parseColor(String sColor)
     {
	String sColors[] = {"white", "green", "red", "blue", "yellow", "purple", "orange", "magenta", "pink", "gray20"};
	
	Color Colors[] = {
	     Color.white, Color.green, Color.red, Color.blue, Color.yellow, new Color((float)0.7, (float)0.3, (float)1.0), new Color((float)1.0, (float)0.7, (float)0.0), Color.magenta, Color.pink, new Color((float)0.2, (float)0.2, (float)0.2)
	};
	
	for(int i = 0; i < sColors.length; i++)
	  {
	     if(sColor.compareToIgnoreCase(sColors[i]) == 0)
	       return(Colors[i]);
	  }
	
	return(null);
     
     } // End of method parseColor()
   
   /* Method      - newCoord
    * Parameters  - coordinate value as a string
    * Returns     - newly created coordinate object
    * Description - this method creates a coordinate object appropriate for the specified 
    * axis type.
    */
   private Coord newCoord(int type, String s)
     {
	Coord c = null;
	
	switch(type)
	  {
	   case cType.U_INT:
	     c = new cUnsigned(s);
	     break;
	     
	   case cType.INT:
	     c = new cSigned(s);
	     break;
	     
	   case cType.DOUBLE:
	     c = new cDouble(s);
	     break;
	     
	   case cType.TIMEVAL:
	     c = new cTimeVal(s);
	     break;
	     
	   case cType.DTIME:
	     c = new cDTime(s);
	     break;
	     
	   default:
	     Error("Plotter.java {method newCoord(int , String)}: unknown coord type");
	  }
	
	return(c);
   
     } // End of method newCoord
   
   /* Method      - compute Range
    * Parameters  - NULL
    * Returns     - NULL
    * Description - computes the window limits within which the plot is to be displayed.
    */
   private void computeRange()
     {
	try
	  {
	     boolean virgin = true;
	     int size = commands.size();
	     Command cmd = null;
	     for(int i = 0; i < size; i++)
	       {
		  cmd = (Command)commands.get(i);
		  if((cmd.type == cmdType.TITLE)  ||
		     (cmd.type == cmdType.XLABEL) ||
		     (cmd.type == cmdType.YLABEL)
		     )
		    continue;
		  
		  switch(cmd.type)
		    {
		     case cmdType.DLINE     :
		     case cmdType.LINE      :
		       if(virgin || (cmd.xb.compareTo(xLeft[viewNo]) < 0))
			 xLeft[viewNo].set(cmd.xb.get());
		       if(virgin || (cmd.xb.compareTo(xRight[viewNo]) > 0))
			 xRight[viewNo].set(cmd.xb.get());
		       if(virgin || (cmd.yb.compareTo(yBottom[viewNo]) < 0))
			 yBottom[viewNo].set(cmd.yb.get());
		       if(virgin || (cmd.yb.compareTo(yTop[viewNo]) > 0))
			 yTop[viewNo].set(cmd.yb.get());
		       virgin = false;
		     case cmdType.X         :
		     case cmdType.DOT       :
		     case cmdType.PLUS      :
		     case cmdType.BOX       :
		     case cmdType.DIAMOND   :
		     case cmdType.UTICK     :
		     case cmdType.DTICK     :
		     case cmdType.LTICK     :
		     case cmdType.RTICK     :
		     case cmdType.HTICK     :
		     case cmdType.VTICK     :
		     case cmdType.UARROW    :
		     case cmdType.DARROW    :
		     case cmdType.LARROW    :
		     case cmdType.RARROW    :
		     case cmdType.INVISIBLE :
		     case cmdType.TEXT      :
		       if(virgin || (cmd.xa.compareTo(xLeft[viewNo]) < 0))
			 xLeft[viewNo].set(cmd.xa.get());
		       if(virgin || (cmd.xa.compareTo(xRight[viewNo]) > 0))
			 xRight[viewNo].set(cmd.xa.get());
		       if(virgin || (cmd.ya.compareTo(yBottom[viewNo]) < 0))
			 yBottom[viewNo].set(cmd.ya.get());
		       if(virgin || (cmd.ya.compareTo(yTop[viewNo]) > 0))
			 yTop[viewNo].set(cmd.ya.get());
		       virgin = false;
		       break;	     
		     default                :
		       Error("jPlot: {DrawPlot.java: computeRange()}: unknown command type");
		    }
	       }

	     /* This is to fix the window so that the labels can be seen correctly.
	      * (In xplot the labels would get clipped-off the edge, if they were at end points in the graph).
	      * Just add 10% to the existing range.
	      */ 
	     if(isMargin)
	       {
		  xLeft[viewNo].set(xLeft[viewNo].get() - Math.rint((xRight[viewNo].get() - xLeft[viewNo].get()) * Constant.MARGIN));
		  xRight[viewNo].set(xRight[viewNo].get() + Math.rint((xRight[viewNo].get() - xLeft[viewNo].get()) * Constant.MARGIN));
		  yBottom[viewNo].set(yBottom[viewNo].get() - Math.rint((yTop[viewNo].get() - yBottom[viewNo].get()) * Constant.MARGIN));
		  yTop[viewNo].set(yTop[viewNo].get() + Math.rint((yTop[viewNo].get() - yBottom[viewNo].get()) * Constant.MARGIN));
	       }
	  }
	catch(IndexOutOfBoundsException e)
	  {
	     System.out.println(e.toString());
	  }
	
     } // End of method computeRange()


   /* Method      - bumpCoord
    * Parameters  - axis type and the coordinate to bump
    * Returns     - NULL
    * Description - This method bumps the coordinates of the axis to the correct range values, in order to
    * get the axis ticks correct. (Used in conjunction with the "tick" method)
    * NOTES       - This algorithm is taken as-is from Tim Shepard's xplot.
    */
   private void bumpCoord(int type, Coord c)
     {
	Coord cCopy = null;   // temporary coordinates used to bump coordinate c
	Coord     t = null;
	int level = 0;        // level used to compute tick marks
	
	switch(type)
	  {
	   case cType.U_INT:
	     t     = new cUnsigned();
	     cCopy = new cUnsigned(c); 
	     break;
	     
	   case cType.INT:
	     t     = new cSigned();
	     cCopy = new cSigned(c); 
	     break;

	   case cType.DOUBLE:
	     t     = new cDouble();
	     cCopy = new cDouble(c); 
	     break;
	     
	   case cType.TIMEVAL:
	     t     = new cTimeVal();
	     cCopy = new cTimeVal(c); 
	     break;
	     
	   case cType.DTIME:
	     t     = new cDTime();
	     cCopy = new cDTime(c); 
	     break;
	     
	   default:
	     Error("DrawPlot.java {method bumpCoord(int , Coord)}: unknown coord type");
	  }
	
	do
	  {
	     t.tick(level++);
	     c.add(t);
	  } while(c.compareTo(cCopy) == 0);
     
     } // End of method bumpCoord()

   
   /* Method      - makeAxis
    * Parameters  - NULL
    * Returns     - NULL
    * Description - creates the axis for this plotter. (called from DrawPlot.java: {method sizeWindow()})
    */
   public void makeAxis()
     {
	Command cmd = null;
	Color     c = null;
	Coord    x1 = newCoord(xType, "0");
	Coord    y1 = newCoord(yType, "0");
	Coord    x2 = newCoord(xType, "0");
	Coord    y2 = newCoord(yType, "0");
	
	x1.set(xLeft[viewNo].get())  ;  y1.set(yTop[viewNo].get());
	x2.set(xLeft[viewNo].get())  ;  y2.set(yBottom[viewNo].get());
	/* arguments for this constructor
	 * Command(type, position, mappped, decoration, color, coord1, coord2, text)
	 */ 
	cmd = new Command(cmdType.LINE, Position.UNDEFINED, false, true, c, x1, y1, x2, y2, "");
	commands.add(0, cmd);

	/* New coordinates are created since we would land up modifying the x-axis values (above), if modified directly */
	x1 = newCoord(xType, "0");
	y1 = newCoord(yType, "0");
	x2 = newCoord(xType, "0");
	y2 = newCoord(yType, "0");

	x1.set(xLeft[viewNo].get())  ;  y1.set(yBottom[viewNo].get());
	x2.set(xRight[viewNo].get()) ;  y2.set(yBottom[viewNo].get());
	/* arguments for this constructor
	 * Command(type, position, mappped, decoration, color, coord1, coord2, text)
	 */ 
	cmd = new Command(cmdType.LINE, Position.UNDEFINED, false, true, c, x1, y1, x2, y2, "");
	commands.add(0, cmd);
	
	/* Set the tick marks on the axis */
	setTicks(xType, xLeft[viewNo], xRight[viewNo], true);   // flag indicates horizontal axis
	setTicks(yType, yBottom[viewNo], yTop[viewNo], false);  // flag indicates vertical axis, no horizontal

     } // End of method makeAxis()
   
   
   /* Method      - setTicks
    * Parameters  - begining and end of the axis, and a flag indicating whether it is horizontal/vertical axis.
    * Returns     - NULL
    * Description - this method takes the begining and end of the axis and sets the appropriate tick marks on it.
    * It is smart enough to compute the number of ticks that may fit, and set them with the appropriate tick labels.
    * NOTES       - this algorithm has been taken as-is from Tim Shepard's xplot.
    */
   public void setTicks(int type, Coord first, Coord last, boolean horizontal)
     {
	int level             = 0;
	int subLevel          = 0;
	int count             = 0;        
	int subCount          = 0; 
	int maxExtraSubticks  = 0; 
	Coord at   = newCoord(type, "0");
	Coord step = newCoord(type, "0");
	
	if(horizontal)
	  maxExtraSubticks = Constant.MAXTICKS + 1;
	else
	  maxExtraSubticks = (Constant.MAXTICKS + 1) * 2;

	/* Start with the smallest tick level */
	for(level = 0; ; level++)
	  {
	     step.tick(level);
	     at.roundUp(first, step);
	   
	     for (count=1; count <= Constant.MAXTICKS; count++)
	       {
		  at.add(step);
		  if (at.compareTo(last) > 0)
		    break;
	       } /* count is now number of ticks that fit */
	   
	     /* if <=  max allowed, use this level */
	     if(count <= Constant.MAXTICKS) break;
	  }
	
	step.tick(level);
	at.roundUp(first, step);
	
	while(at.compareTo(last) <= 0)
	  {
	     if(horizontal)         // flag indicates labels are to be placed with ticks
	       doXTick(at, true);
	     else
	       doYTick(at, true);
	     
	     at.add(step);
	  }
	
	subLevel = first.subTick(level);
	if(subLevel != level)
	  {
	     step.tick(subLevel);
	     at.roundUp(first, step);
	   
	     for (subCount = 1; subCount - count <= maxExtraSubticks; subCount++)
	       {
		  at.add(step);
		  if(at.compareTo(last) > 0)
		    break;
	       }
	     
	     if(subCount - count <= maxExtraSubticks)
	       {
		  at.roundUp(first, step);
		  while(at.compareTo(last) <= 0)
		    {
		       if(horizontal)         // flag indicates labels are NOT to be placed with ticks
			 doXTick(at, false);
		       else
			 doYTick(at, false);

		       at.add(step);
		    }
	       }
	  }
	
     } // End of method setTicks()
   
   /* Method      - doXTick
    * Parameters  - tick position and a flag indicating whether a label is to be placed with the tick.
    * (labels are not placed with subticks).
    * Returns     - NULL
    * Description - creates a new command that places a tick at the specified position, and possibly a label.
    */
   private void doXTick(Coord r, boolean labelFlag)
     {
	Command cmd = null;
	Color     c = null;
	Coord    x1 = newCoord(xType, "0");
	Coord    y1 = newCoord(yType, "0");

	x1.set(r.get())  ;  y1.set(yBottom[viewNo].get());
	/* arguments for this constructor
	 * Command(type, position, mappped, decoration, color, coord1, coord2, text)
	 */ 
	cmd = new Command(cmdType.DTICK, Position.UNDEFINED, false, true, c, x1, y1, null, null, "");
	commands.add(0, cmd);
	
	if(labelFlag == true)
	  {
	     String text = r.unparse();
	     cmd = new Command(cmdType.TEXT, Position.BELOW, false, true, c, x1, y1, null, null, (text + " " + xUnits));
	     commands.add(0, cmd);
	  }

     } // End of method doXTick()
   
   /* Method      - doYTick
    * Parameters  - tick position and a flag indicating whether a label is to be placed with the tick.
    * (labels are not placed with subticks).
    * Returns     - NULL
    * Description - creates a new command that places a tick at the specified position, and possibly a label.
    */
   private void doYTick(Coord r, boolean labelFlag)
     {
	Command cmd = null;
	Color     c = null;
	Coord    x1 = newCoord(xType, "0");
	Coord    y1 = newCoord(yType, "0");

	x1.set(xLeft[viewNo].get())  ;  y1.set(r.get());
	/* arguments for this constructor
	 * Command(type, position, mappped, decoration, color, coord1, coord2, text)
	 */ 
	cmd = new Command(cmdType.LTICK, Position.UNDEFINED, false, true, c, x1, y1, null, null, "");
	commands.add(0, cmd);
	
	if(labelFlag == true)
	  {
	     String text = r.unparse();
	     cmd = new Command(cmdType.TEXT, Position.LEFT, false, true, c, x1, y1, null, null, (text + " " + yUnits));
	     commands.add(0, cmd);
	  }
	
     } // End of method doYTick()

   /* Method      - reSync
    * Parameters  - NULL
    * Returns     - NULL
    * Description - resyncs the plotter data with the current state of the file. Basically re-reads the
    * data from the file.
    */
   public void reSync()
     {
	/* Initializations */
	commands        = new ArrayList();
	xUnits          = new String("");
	yUnits          = new String("");
	aspectRatio     = 0.0;
	viewNo          = 0;
	xLeft           = new Coord[Constant.NUMVIEWS];
	yBottom         = new Coord[Constant.NUMVIEWS];
	xRight          = new Coord[Constant.NUMVIEWS];
	yTop            = new Coord[Constant.NUMVIEWS];
	origin          = new Point2D.Double(Constant.DEFAULT_ORIGIN_X, Constant.DEFAULT_ORIGIN_Y);
	dragStart       = new Point(0, 0);
	dragEnd         = new Point(0, 0);
	defaultColor    = Color.white;
	currentColor    = Color.white;
	foregroundColor = Color.white;
	backgroundColor = Color.black;
	state           = State.NORMAL;
	this.isMargin   = isMargin;
	/* Read the commands from the file */
	readInput();
     

     } // End of method reSync()
   
   /* Method      - Error
    * Parameters  - error message
    * Returns     - NULL
    * Description - prints the error message and exits.
    */
   private void Error(String msg)
     {
	System.out.println(msg);
	System.exit(1);
     }
   
} // End of class Plotter
